@@ -1,7 +1,7 @@
 #     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
-""" Locating modules and package source on disk.
+"""Locating modules and package source on disk.
 
 The actual import of a module would already execute code that changes things.
 Imagine a module that does ``os.system()``, it would be done during
@@ -26,20 +26,34 @@ import os
 import sys
 import zipfile
 
-from nuitka import SourceCodeReferences
 from nuitka.__past__ import iter_modules
 from nuitka.containers.Namedtuples import makeNamedtupleClass
 from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.Errors import NuitkaCodeDeficit
-from nuitka.plugins.Plugins import Plugins
-from nuitka.PythonFlavors import isNuitkaPython
+from nuitka.options.Options import (
+    getMainEntryPointFilenames,
+    getOutputFolderName,
+    hasPythonFlagNoCurrentDirectoryInPath,
+    shallExplainImports,
+)
+from nuitka.OutputDirectories import getSourceDirectoryPath
+from nuitka.plugins.Hooks import (
+    decideRecompileExtensionModules,
+    getPackageExtraScanPaths,
+    suppressUnknownImportWarning,
+)
+from nuitka.PythonFlavors import isMonolithPy
 from nuitka.PythonVersions import python_version
+from nuitka.SourceCodeReferences import makeSourceReferenceFromFilename
+from nuitka.States import states
 from nuitka.Tracing import recursion_logger
 from nuitka.tree.ReformulationMultidist import locateMultidistModule
 from nuitka.utils.AppDirs import getCacheDir
 from nuitka.utils.FileOperations import (
+    areSamePaths,
     getFileContentsHash,
     getNormalizedPath,
+    getNormalizedPathJoin,
     listDir,
     removeDirectory,
 )
@@ -70,35 +84,37 @@ _debug_module_finding = None
 # Do not add current directory to search path.
 _safe_path = None
 
+
 # Debug mode
-_is_debug = None
-
-
 def setupImportingFromOptions():
     """Set up the importing layer from giving options."""
 
-    # Should only be used inside of here.
-    from nuitka import Options
-
     # singleton, pylint: disable=global-statement
     global _debug_module_finding
-    _debug_module_finding = Options.shallExplainImports()
+    _debug_module_finding = shallExplainImports()
 
     global _safe_path
-    _safe_path = Options.hasPythonFlagNoCurrentDirectoryInPath()
-
-    global _is_debug
-    _is_debug = Options.is_debug
+    _safe_path = hasPythonFlagNoCurrentDirectoryInPath()
 
     # Lets try and have this complete, please report failures.
-    if _is_debug and not isNuitkaPython():
+    if states.is_debug and not isMonolithPy():
         _checkRaisingBuiltinComplete()
 
-    main_filenames = Options.getMainEntryPointFilenames()
-    for filename in main_filenames:
+    if getOutputFolderName() is not None:
+        source_dir = getSourceDirectoryPath(onefile=False, create=False)
+    else:
+        source_dir = None
+
+    for filename in getMainEntryPointFilenames():
         # Inform the importing layer about the main script directory, so it can use
         # it when attempting to follow imports.
-        addMainScriptDirectory(main_dir=os.path.dirname(os.path.abspath(filename)))
+
+        main_dir = os.path.dirname(os.path.abspath(filename))
+
+        if source_dir is not None and areSamePaths(source_dir, main_dir):
+            continue
+
+        addMainScriptDirectory(main_dir)
 
 
 def _checkRaisingBuiltinComplete():
@@ -302,7 +318,7 @@ def getModuleNameAndKindFromFilename(module_filename):
 
 
 def isIgnoreListedImportMaker(source_ref):
-    if isNuitkaPython():
+    if isMonolithPy():
         return True
 
     return isStandardLibraryPath(source_ref.getFilename())
@@ -321,7 +337,7 @@ def warnAboutNotFoundImport(importing, module_name, level, source_ref):
         if key not in warned_about:
             warned_about.add(key)
 
-            if Plugins.suppressUnknownImportWarning(
+            if suppressUnknownImportWarning(
                 importing=importing, source_ref=source_ref, module_name=module_name
             ):
                 return
@@ -554,7 +570,7 @@ def _findModuleInPath3(
         for suffix, module_type in getModuleFilenameSuffixes():
             package_file_name = "__init__" + suffix
 
-            file_path = os.path.join(package_directory, package_file_name)
+            file_path = getNormalizedPathJoin(package_directory, package_file_name)
 
             if os.path.isfile(file_path):
                 yield (
@@ -593,7 +609,7 @@ def _findModuleInPath3(
             if module_type == last_module_type:
                 continue
 
-            full_path = os.path.join(search_path_entry, module_name + suffix)
+            full_path = getNormalizedPathJoin(search_path_entry, module_name + suffix)
 
             if os.path.isfile(full_path):
                 yield (
@@ -617,7 +633,9 @@ def _getSetuptoolsDistutilsPackageDir():
     )[1]
 
     if setuptools_package_dir is not None:
-        setuptools_package_dir = os.path.join(setuptools_package_dir, "_distutils")
+        setuptools_package_dir = getNormalizedPathJoin(
+            setuptools_package_dir, "_distutils"
+        )
 
     return setuptools_package_dir
 
@@ -652,7 +670,9 @@ def _findModuleInPath2(package_name, module_name, search_path, logger):
             continue
         considered.add(os.path.normcase(search_path_entry))
 
-        package_directory = os.path.join(search_path_entry, module_name.asPath())
+        package_directory = getNormalizedPathJoin(
+            search_path_entry, module_name.asPath()
+        )
 
         for candidate in _findModuleInPath3(
             module_name=module_name,
@@ -691,9 +711,7 @@ def _findModuleInPath2(package_name, module_name, search_path, logger):
 
             def prioritize(c):
                 if c.module_type == "PY_SOURCE":
-                    decision, reason = Plugins.decideRecompileExtensionModules(
-                        c.found_as
-                    )
+                    decision, reason = decideRecompileExtensionModules(c.found_as)
                     _recompile_extension_modules[c.found_as] = decision, reason
                 else:
                     decision = False
@@ -763,7 +781,7 @@ def _unpackPathElement(path_entry):
             if path_entry not in _egg_files:
                 checksum = getFileContentsHash(path_entry)
 
-                target_dir = os.path.join(getCacheDir("egg-content"), checksum)
+                target_dir = getNormalizedPathJoin(getCacheDir("egg-content"), checksum)
 
                 if not os.path.exists(target_dir):
                     try:
@@ -813,7 +831,7 @@ def getPackageSearchPath(package_name):
 
         result = []
         for element in getPackageSearchPath(parent_package_name):
-            package_dir = os.path.join(element, child_package_name.asPath())
+            package_dir = getNormalizedPathJoin(element, child_package_name.asPath())
 
             if isPackageDir(package_dir):
                 result.append(package_dir)
@@ -832,9 +850,9 @@ def getPackageSearchPath(package_name):
             return preloaded_path
 
         def getPackageDirCandidates(element):
-            yield os.path.join(element, package_name.asPath()), False
+            yield getNormalizedPathJoin(element, package_name.asPath()), False
 
-            for extra_path in Plugins.getPackageExtraScanPaths(package_name, element):
+            for extra_path in getPackageExtraScanPaths(package_name, element):
                 yield extra_path, True
 
         result = []
@@ -1003,7 +1021,7 @@ def locateModule(module_name, parent_package, level, logger=None):
                 module_name, module_package
             )
 
-            if _is_debug:
+            if states.is_debug:
                 assert module_name == found_module_name, (
                     module_name,
                     found_module_name,
@@ -1061,7 +1079,7 @@ def decideModuleSourceRef(filename, module_name, is_main, is_fake, logger):
     is_package = False
 
     if is_main and os.path.isdir(filename):
-        source_filename = getNormalizedPath(os.path.join(filename, "__main__.py"))
+        source_filename = getNormalizedPathJoin(filename, "__main__.py")
 
         if not os.path.isfile(source_filename):
             sys.stderr.write(
@@ -1079,32 +1097,30 @@ def decideModuleSourceRef(filename, module_name, is_main, is_fake, logger):
     if is_fake:
         source_filename = filename
 
-        source_ref = SourceCodeReferences.fromFilename(filename=filename)
+        source_ref = makeSourceReferenceFromFilename(filename=filename)
 
         module_name = is_fake
 
     elif os.path.isfile(filename):
         source_filename = filename
 
-        source_ref = SourceCodeReferences.fromFilename(filename=filename)
+        source_ref = makeSourceReferenceFromFilename(filename=filename)
 
     elif isPackageDir(filename):
         is_package = True
 
-        source_filename = getNormalizedPath(os.path.join(filename, "__init__.py"))
+        source_filename = getNormalizedPathJoin(filename, "__init__.py")
 
         if not os.path.isfile(source_filename):
-            source_ref = SourceCodeReferences.fromFilename(
-                filename=filename
-            ).atInternal()
+            source_ref = makeSourceReferenceFromFilename(filename=filename).atInternal()
             is_namespace = True
         else:
-            source_ref = SourceCodeReferences.fromFilename(
+            source_ref = makeSourceReferenceFromFilename(
                 filename=getNormalizedPath(os.path.abspath(source_filename))
             )
 
     else:
-        logger.sysexit(
+        return logger.sysexit(
             "%s: can't open file '%s'." % (os.path.basename(sys.argv[0]), filename),
             exit_code=2,
         )
@@ -1121,7 +1137,7 @@ def decideModuleSourceRef(filename, module_name, is_main, is_fake, logger):
 # spell-checker: ignore _posixsubprocess,pyexpat,xxsubtype,_bytesio,_interpchannels
 # spell-checker: ignore _interpqueues,_lsprof,_multibytecodec,_posixshmem _winapi
 # spell-checker: ignore  _testbuffer _testexternalinspection _testimportmultiple
-# spell-checker: ignore _testinternalcapi _testmultiphase _testsinglephase
+# spell-checker: ignore _testinternalcapi _testmultiphase _testsinglephase _scproxy
 # spell-checker: ignore _xxtestfuzz _xxsubinterpreters imageop _xxinterpchannels
 _stdlib_module_raises = {
     "_abc": False,
@@ -1300,11 +1316,11 @@ def getChildPackageNames(module_name):
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     Licensed under the Apache License, Version 2.0 (the "License");
+#     Licensed under the GNU Affero General Public License, Version 3 (the "License");
 #     you may not use this file except in compliance with the License.
 #     You may obtain a copy of the License at
 #
-#        http://www.apache.org/licenses/LICENSE-2.0
+#        http://www.gnu.org/licenses/agpl.txt
 #
 #     Unless required by applicable law or agreed to in writing, software
 #     distributed under the License is distributed on an "AS IS" BASIS,
