@@ -1,7 +1,7 @@
 #     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
-""" Standard plug-in to tell Nuitka about DLLs needed for standalone imports.
+"""Standard plug-in to tell Nuitka about DLLs needed for standalone imports.
 
 When DLLs are imported, we cannot see this and need to be told that. This
 encodes the knowledge we have for various modules. Feel free to add to this
@@ -12,7 +12,7 @@ import fnmatch
 import os
 import sys
 
-from nuitka.Options import isStandaloneMode
+from nuitka.options.Options import isStandaloneMode
 from nuitka.plugins.YamlPluginBase import NuitkaYamlPluginBase
 from nuitka.PythonVersions import python_version
 from nuitka.utils.Distributions import (
@@ -31,7 +31,7 @@ from nuitka.utils.Utils import isFreeBSD, isLinux, isMacOS, isWin32Windows
 class NuitkaPluginDllFiles(NuitkaYamlPluginBase):
     plugin_name = "dll-files"
     plugin_desc = "Include DLLs as per package configuration files."
-    plugin_category = "core"
+    plugin_category = "core,feature"
 
     @staticmethod
     def isAlwaysEnabled():
@@ -363,8 +363,10 @@ conditions are missing, or this version of the module needs treatment added."""
                     reason="needed by uuid package",
                 )
         elif full_name == "iptc" and isLinux():
+            # spell-checker: ignore iptc
             import iptc.util  # pylint: disable=I0021,import-error
 
+            # spell-checker: ignore xtwrapper
             xtwrapper_dll = iptc.util.find_library("xtwrapper")[0]
             xtwrapper_dll_path = xtwrapper_dll._name  # pylint: disable=protected-access
 
@@ -375,7 +377,7 @@ conditions are missing, or this version of the module needs treatment added."""
                 package_name=None,
                 reason="needed by 'iptc'",
             )
-        # TODO: This should be its own plugin.
+        # TODO: This should be its own plugin, spell-checker: ignore pythoncom,win32evtlog,win32wnet
         elif (
             full_name
             in (
@@ -413,6 +415,7 @@ conditions are missing, or this version of the module needs treatment added."""
             pywin_dir = getPyWin32Dir()
 
             if pywin_dir is not None:
+                # spell-checker: ignore pywintypes
                 for dll_name in "pythoncom", "pywintypes":
                     pythoncom_filename = "%s%d%d.dll" % (
                         dll_name,
@@ -529,11 +532,11 @@ conditions are missing, or this version of the module needs treatment added."""
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     Licensed under the Apache License, Version 2.0 (the "License");
+#     Licensed under the GNU Affero General Public License, Version 3 (the "License");
 #     you may not use this file except in compliance with the License.
 #     You may obtain a copy of the License at
 #
-#        http://www.apache.org/licenses/LICENSE-2.0
+#        http://www.gnu.org/licenses/agpl.txt
 #
 #     Unless required by applicable law or agreed to in writing, software
 #     distributed under the License is distributed on an "AS IS" BASIS,
