@@ -1,7 +1,7 @@
 #     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
-""" Utils for file and directory operations.
+"""Utils for file and directory operations.
 
 This provides enhanced and more error resilient forms of standard
 stuff. It will also frequently add sorting for determinism.
@@ -76,6 +76,41 @@ def withFileLock(reason="unknown"):
     file_lock.release()
 
 
+@contextmanager
+def withTemporaryDirectory(
+    logger,
+    prefix=None,
+    suffix=None,
+    directory=None,
+    ignore_errors=False,
+    extra_recommendation=None,
+):
+    """Create a temporary directory and remove it afterwards.
+
+    Args:
+        logger: Logger to use for error reporting.
+        prefix: Prefix of the directory name.
+        suffix: Suffix of the directory name.
+        directory: Directory where to create it.
+        ignore_errors: Ignore errors when removing the directory.
+        extra_recommendation: Recommendation what to do if removal fails.
+
+    Yields:
+        The path of the temporary directory.
+    """
+    temp_dir = tempfile.mkdtemp(prefix=prefix, suffix=suffix, dir=directory)
+
+    try:
+        yield temp_dir
+    finally:
+        removeDirectory(
+            temp_dir,
+            logger=logger,
+            ignore_errors=ignore_errors,
+            extra_recommendation=extra_recommendation,
+        )
+
+
 def areSamePaths(path1, path2):
     """Decide if two paths the same.
 
@@ -103,7 +138,13 @@ def areSamePaths(path1, path2):
     if path1 == path2:
         return True
 
-    if os.path.exists(path1) and os.path.exists(path2):
+    exists1 = os.path.exists(path1)
+    exists2 = os.path.exists(path2)
+
+    if exists1 != exists2:
+        return False
+
+    if exists1 and exists2:
         path1 = getExternalUsePath(path1)
         path2 = getExternalUsePath(path2)
 
@@ -127,6 +168,15 @@ def areInSamePaths(path1, path2):
 
 
 def haveSameFileContents(path1, path2):
+    """Check if two files have the same contents.
+
+    Args:
+        path1: First path
+        path2: Second path
+
+    Returns:
+        bool: True if files have same content.
+    """
     # Local import, to avoid this for normal use cases.
     import filecmp
 
@@ -134,9 +184,29 @@ def haveSameFileContents(path1, path2):
 
 
 def getFileSize(path):
+    """Get the size of a file.
+
+    Args:
+        path: File path
+
+    Returns:
+        int: File size in bytes.
+    """
     return os.path.getsize(path)
 
 
+def getFileModificationTime(path):
+    """Get the modification time of a file.
+
+    Args:
+        path: File path
+
+    Returns:
+        float: File modification time.
+    """
+    return os.stat(path).st_mtime
+
+
 def relpath(path, start="."):
     """Make it a relative path, if possible.
 
@@ -167,6 +237,14 @@ def relpath(path, start="."):
 
 
 def isRelativePath(path):
+    """Check if a path is relative.
+
+    Args:
+        path: Path to check
+
+    Returns:
+        bool: True if path is relative.
+    """
     if os.path.isabs(path):
         return False
     if path.startswith((".." + os.path.sep, "../")):
@@ -175,6 +253,41 @@ def isRelativePath(path):
     return True
 
 
+def cheapCopyFile(source_path, dest_path):
+    makeContainingPath(dest_path)
+
+    if isWin32Windows():
+        # Windows has symlinks these days, but they do not integrate well
+        # with Python2 at least. So make a copy in any case.
+        deleteFile(dest_path, must_exist=False)
+        copyFile(source_path, dest_path)
+    else:
+        # Relative paths work badly for links. Creating them relative is
+        # not worth the effort.
+        src = os.path.abspath(source_path)
+
+        try:
+            link_target = os.readlink(dest_path)
+
+            # If it's already a proper link, do nothing then.
+            if link_target == src:
+                return
+
+            deleteFile(dest_path, must_exist=True)
+        except OSError as _e:
+            # Broken links work like that, remove them, so we can replace
+            # them.
+            try:
+                deleteFile(dest_path, must_exist=False)
+            except OSError:
+                pass
+
+        try:
+            os.symlink(src, dest_path)
+        except OSError:
+            copyFile(src, dest_path)
+
+
 def makePath(path):
     """Create a directory if it doesn't exist.
 
@@ -197,6 +310,11 @@ def makePath(path):
 
 
 def makeContainingPath(filename):
+    """Create the directory for containing a file if it doesn't exist.
+
+    Args:
+        filename: Path to the file.
+    """
     target_dir = os.path.dirname(filename)
 
     if not os.path.isdir(target_dir):
@@ -204,10 +322,47 @@ def makeContainingPath(filename):
 
 
 def isPathExecutable(path):
-    """Is the given path executable."""
+    """Check if the given path is executable.
+
+    Args:
+        path: Path to check.
+
+    Returns:
+        bool: True if executable.
+    """
     return os.path.isfile(path) and os.access(path, os.X_OK)
 
 
+def hasDirectoryContents(path):
+    """Check if a directory has contents.
+
+    Args:
+        path: directory to check
+
+    Returns:
+        bool: True if directory exists and has contents, False otherwise.
+    """
+    if not os.path.isdir(path):
+        return False
+
+    if python_version >= 0x350:
+        try:
+            scan = os.scandir(path)
+            result = next(scan, None) is not None
+        except OSError:
+            return False
+
+        if python_version >= 0x360:
+            scan.close()
+
+        return result
+    else:
+        try:
+            return bool(os.listdir(path))
+        except OSError:
+            return False
+
+
 # Make sure we don't repeat this too much.
 _real_path_windows_cache = {}
 _powershell_path = None
@@ -248,7 +403,7 @@ def _getRealPathWindows(path):
                 # Python versions, many tools won't work with those.
                 _real_path_windows_cache[path] = path
             else:
-                _real_path_windows_cache[path] = os.path.join(
+                _real_path_windows_cache[path] = getNormalizedPathJoin(
                     os.path.dirname(path), result.rstrip("\r\n")
                 )
         else:
@@ -301,13 +456,13 @@ def _restoreWindowsPath(orig_path, path):
             if dirname:
                 dirname = getDirectoryRealPath(dirname)
 
-                # Drive letters do not get slashes from "os.path.join", so
+                # Drive letters do not get slashes from 'os.path.join', so
                 # we inject this here and normalize the path afterwards to
                 # remove any duplication added.
                 if os.path.sep not in dirname:
                     dirname = dirname + os.path.sep
 
-                path = os.path.normpath(os.path.join(dirname, filename))
+                path = getNormalizedPathJoin(dirname, filename)
 
     return path
 
@@ -354,32 +509,27 @@ def listDir(path):
         calling side.
 
         This should be used, because it makes sure to resolve the
-        symlinks to directories on Windows, that a naive "os.listdir"
+        symlinks to directories on Windows, that a naive 'os.listdir'
         won't do by default.
     """
     real_path = getDirectoryRealPath(path)
 
-    # The "os.listdir" output needs to be unicode paths, or else it can be unusable
+    # The 'os.listdir' output needs to be unicode paths, or else it can be unusable
     # for Python2 on Windows at least. We try to go back on the result.
     if str is bytes and type(real_path) is str:
         real_path = unicode(real_path)
 
-    def _tryDecodeToStr(value):
-        if str is bytes:
-            if type(value) is unicode:
-                # File system paths, that should be usable for names of modules,
-                # as Python2 code objects will e.g. hate unicode values.
-
-                # spell-checker: ignore getfilesystemencoding
-                try:
-                    return value.decode(sys.getfilesystemencoding())
-                except UnicodeDecodeError:
-                    return value
-        else:
+    def _tryEncodeToFilesystemEncoding(value):
+        try:
+            return encodeToFilesystemEncoding(value)
+        except UnicodeEncodeError:
             return value
 
     return sorted(
-        (_tryDecodeToStr(os.path.join(path, filename)), _tryDecodeToStr(filename))
+        (
+            _tryEncodeToFilesystemEncoding(getNormalizedPathJoin(path, filename)),
+            _tryEncodeToFilesystemEncoding(filename),
+        )
         for filename in os.listdir(real_path)
     )
 
@@ -442,7 +592,7 @@ def getFileList(
             if only_suffixes and not os.path.normcase(filename).endswith(only_suffixes):
                 continue
 
-            fullname = os.path.join(root, filename)
+            fullname = getNormalizedPathJoin(root, filename)
 
             if normalize:
                 fullname = getNormalizedPath(fullname)
@@ -482,13 +632,21 @@ def getSubDirectories(path, ignore_dirs=()):
         dirnames.sort()
 
         for dirname in dirnames:
-            result.append(os.path.join(root, dirname))
+            result.append(getNormalizedPathJoin(root, dirname))
 
     result.sort()
     return result
 
 
 def getDllBasename(path):
+    """Get the basename of a DLL file, stripping version and extension.
+
+    Args:
+        path: Path to DLL file.
+
+    Returns:
+        str or None: Basename of DLL or None if not a DLL.
+    """
     compare_path = os.path.normcase(path)
 
     for suffix in (".dll", ".so", ".dylib"):
@@ -621,6 +779,14 @@ def _isMacOSFramework(path):
 
 
 def isLink(path):
+    """Check if a path is a symbolic link (or junction on Windows).
+
+    Args:
+        path: Path to check.
+
+    Returns:
+        bool: True if path is a link.
+    """
     result = os.path.islink(path)
 
     # Special handling for Junctions.
@@ -664,7 +830,15 @@ def deleteFile(path, must_exist):
 
 
 def searchPrefixPath(path, element):
-    """Search element and return prefix in path, if any."""
+    """Search element and return prefix in path, if any.
+
+    Args:
+        path: Path to search in.
+        element: Element to search for.
+
+    Returns:
+         str or None: Prefix path if found, None otherwise.
+    """
 
     while path:
         if os.path.normcase(os.path.basename(path)) == os.path.normcase(element):
@@ -691,17 +865,40 @@ def getFilenameExtension(path):
 
 
 def changeFilenameExtension(path, extension):
-    """Change the filename extension."""
+    """Change the filename extension.
+
+    Args:
+        path: Path to change.
+        extension: New extension (including dot).
+
+    Returns:
+        str: Path with new extension.
+    """
 
     is_legal, illegal_reason = isLegalPath(path)
     if not is_legal:
         raise NuitkaFilenameError(illegal_reason)
 
-    return os.path.splitext(path)[0] + extension
+    result = os.path.splitext(path)[0] + extension
+
+    is_legal, illegal_reason = isLegalPath(result)
+    if not is_legal:
+        raise NuitkaFilenameError(illegal_reason)
+
+    return result
 
 
 def switchFilenameExtension(path, old_extension, new_extension):
-    """Switch the filename extension specified to another one."""
+    """Switch the filename extension specified to another one.
+
+    Args:
+        path: Path to check.
+        old_extension: Extension expected.
+        new_extension: Extension to switch to.
+
+    Returns:
+        str: Path with switched extension.
+    """
     assert path.endswith(old_extension)
     return path[: -len(old_extension)] + new_extension
 
@@ -723,6 +920,15 @@ def hasFilenameExtension(path, extensions):
 
 
 def addFilenameExtension(path, extension):
+    """Add an extension to a filename if it doesn't already have it.
+
+    Args:
+        path: Path to add extension to.
+        extension: Extension to add (including dot).
+
+    Returns:
+        str: Path with extension added if missing.
+    """
     if not hasFilenameExtension(path, extension):
         path += extension
 
@@ -780,6 +986,14 @@ def removeDirectory(path, logger, ignore_errors, extra_recommendation):
 
 
 def resetDirectory(path, logger, ignore_errors, extra_recommendation):
+    """Reset a directory, removing it if it exists and recreating it empty.
+
+    Args:
+        path: Path to reset.
+        logger: Logger for output.
+        ignore_errors: Whether to ignore errors during removal.
+        extra_recommendation: Recommendation message for errors.
+    """
     removeDirectory(
         path=path,
         logger=logger,
@@ -791,7 +1005,18 @@ def resetDirectory(path, logger, ignore_errors, extra_recommendation):
 
 @contextmanager
 def withTemporaryFile(prefix="", suffix="", mode="w", delete=True, temp_path=None):
-    """Provide a temporary file opened and potentially deleted."""
+    """Provide a temporary file opened and potentially deleted.
+
+    Args:
+        prefix: Filename prefix.
+        suffix: Filename suffix.
+        mode: Open mode.
+        delete: Whether to delete the file on exit.
+        temp_path: Directory for temporary file.
+
+    Yields:
+        file: The temporary file object.
+    """
     with tempfile.NamedTemporaryFile(
         prefix=prefix, suffix=suffix, mode=mode, delete=delete, dir=temp_path
     ) as temp_file:
@@ -800,7 +1025,16 @@ def withTemporaryFile(prefix="", suffix="", mode="w", delete=True, temp_path=Non
 
 @contextmanager
 def withTemporaryFilename(prefix="", suffix="", temp_path=None):
-    """Provide a temporary filename."""
+    """Provide a temporary filename.
+
+    Args:
+        prefix: Filename prefix.
+        suffix: Filename suffix.
+        temp_path: Directory for temporary file.
+
+    Yields:
+        str: The temporary filename.
+    """
     with tempfile.NamedTemporaryFile(
         prefix=prefix,
         suffix=suffix,
@@ -816,6 +1050,17 @@ def withTemporaryFilename(prefix="", suffix="", temp_path=None):
 
 
 def getFileContentByLine(filename, mode="r", encoding=None, errors=None):
+    """Get the contents of a file as lines.
+
+    Args:
+        filename: str with the file to be read
+        mode: "r" for str, "rb" for bytes result
+        encoding: optional encoding to used when reading the file, e.g. "utf8"
+        errors: optional error handler decoding the content, as defined in `codecs`
+
+    Returns:
+        list: List of lines.
+    """
     # We read the whole, to keep lock times minimal. We only deal with small
     # files like this normally.
     return getFileContents(
@@ -861,6 +1106,17 @@ def getFileFirstLine(filename, mode="r", encoding=None):
 
 
 def openTextFile(filename, mode, encoding=None, errors=None):
+    """Open a text file with proper handling.
+
+    Args:
+        filename: Path to file.
+        mode: Open mode.
+        encoding: Encoding to use.
+        errors: Error handling.
+
+    Returns:
+        file: Opened file object.
+    """
     # Do not attempt to create files with what we consider
     # illegal filenames.
     if "w" in mode:
@@ -941,21 +1197,33 @@ def changeTextFileContents(filename, contents, encoding=None, compare_only=False
 
 @contextmanager
 def withPreserveFileMode(filenames):
+    """Context manager to preserve file modes.
+
+    Args:
+        filenames: Single filename or list of filenames.
+    """
     if isinstance(filenames, basestring):
         filenames = [filenames]
 
     old_modes = {}
     for filename in filenames:
-        old_modes[filename] = os.stat(filename).st_mode
+        if os.path.exists(filename):
+            old_modes[filename] = os.stat(filename).st_mode
 
     yield
 
     for filename in filenames:
-        os.chmod(filename, old_modes[filename])
+        if filename in old_modes and os.path.exists(filename):
+            os.chmod(filename, old_modes[filename])
 
 
 @contextmanager
 def withMadeWritableFileMode(filenames):
+    """Context manager to make files writable temporarily.
+
+    Args:
+        filenames: Single filename or list of filenames.
+    """
     if isinstance(filenames, basestring):
         filenames = [filenames]
 
@@ -967,6 +1235,11 @@ def withMadeWritableFileMode(filenames):
 
 
 def removeFileExecutablePermission(filename):
+    """Remove executable permission from a file.
+
+    Args:
+        filename: Path to file.
+    """
     old_stat = os.stat(filename)
 
     mode = old_stat.st_mode
@@ -977,6 +1250,11 @@ def removeFileExecutablePermission(filename):
 
 
 def addFileExecutablePermission(filename):
+    """Add executable permission to a file.
+
+    Args:
+        filename: Path to file.
+    """
     old_stat = os.stat(filename)
 
     mode = old_stat.st_mode
@@ -987,6 +1265,12 @@ def addFileExecutablePermission(filename):
 
 
 def renameFile(source_filename, dest_filename):
+    """Rename a file, with failover to copy+delete on Windows.
+
+    Args:
+        source_filename: Source path.
+        dest_filename: Destination path.
+    """
     # There is no way to safely update a file on Windows, but lets
     # try on Linux at least.
     old_stat = os.stat(source_filename)
@@ -1008,7 +1292,7 @@ def copyTree(source_path, dest_path):
         dest_path: where to copy to, may already exist
 
     Notes:
-        This must be used over `shutil.copytree` which has troubles
+        This must be used over 'shutil.copytree' which has troubles
         with existing directories on some Python versions.
     """
     if python_version >= 0x380:
@@ -1024,29 +1308,41 @@ def copyTree(source_path, dest_path):
 
 
 def resolveSymlink(path):
-    """Resolve a symlink, to a relative path."""
+    """Resolve a symlink, to a relative path.
+
+    Args:
+        path: Path to symlink.
+
+    Returns:
+        str: Relative path to link target.
+    """
     link_source_abs = os.path.abspath(path)
     link_target_abs = os.path.abspath(
-        os.path.join(os.path.dirname(path), os.readlink(path))
+        getNormalizedPathJoin(os.path.dirname(path), os.readlink(path))
     )
     link_target_rel = relpath(link_target_abs, os.path.dirname(link_source_abs))
 
     return link_target_rel
 
 
-def copyFileWithPermissions(source_path, dest_path, dist_dir):
-    """Improved version of shutil.copy2 for putting things to dist folder
+def copyFileWithPermissions(source_path, dest_path, target_dir):
+    """Improved version of 'shutil.copy2' for putting things to dist folder.
 
-    File systems might not allow to transfer extended attributes, which we then ignore
-    and only copy permissions.
+    File systems might not allow to transfer extended attributes, which we then
+    ignore and only copy permissions.
+
+    Args:
+        source_path: Source file.
+        dest_path: Destination file.
+        target_dir: Target directory (to check for symlink containment).
     """
 
     if os.path.islink(source_path) and not isWin32Windows():
         link_target_rel = resolveSymlink(source_path)
 
         if isFilenameBelowPath(
-            path=dist_dir,
-            filename=os.path.join(os.path.dirname(dest_path), link_target_rel),
+            path=target_dir,
+            filename=getNormalizedPathJoin(os.path.dirname(dest_path), link_target_rel),
         ):
             os.symlink(link_target_rel, dest_path)
             return
@@ -1066,10 +1362,14 @@ def copyFileWithPermissions(source_path, dest_path, dist_dir):
 
 
 def copyFile(source_path, dest_path):
-    """Improved version of shutil.copy
+    """Improved version of 'shutil.copy'.
 
-    This handles errors with a chance to correct them, e.g. on Windows, files might be
-    locked by running program or virus checkers.
+    This handles errors with a chance to correct them, e.g. on Windows, files
+    might be locked by running program or virus checkers.
+
+    Args:
+        source_path: Source file.
+        dest_path: Destination file.
     """
 
     while 1:
@@ -1098,18 +1398,29 @@ def copyFile(source_path, dest_path):
 
 
 def getWindowsDrive(path):
-    """Windows drive for a given path."""
+    """Windows drive for a given path.
+
+    Args:
+        path: Path to check.
+
+    Returns:
+        str: Drive letter with colon, normalized.
+    """
 
     drive, _ = os.path.splitdrive(os.path.abspath(path))
     return os.path.normcase(drive)
 
 
 def isFilenameBelowPath(path, filename, consider_short=True):
-    """Is a filename inside of a given directory path
+    """Is a filename inside of a given directory path.
 
     Args:
-        path: location to be below
-        filename: candidate being checked
+        path: Location to be below (can be tuple/list of paths).
+        filename: Candidate being checked.
+        consider_short: Whether to consider short paths on Windows.
+
+    Returns:
+        bool: True if filename is below path.
     """
     if type(path) in (tuple, list):
         for p in path:
@@ -1229,7 +1540,7 @@ def getWindowsLongPathName(filename):
             if ctypes.GetLastError() == 5:
                 return filename
 
-            raiseWindowsError("getWindowsLongPathName for %s" % filename)
+            raiseWindowsError("getWindowsLongPathName for '%s'" % filename)
         if output_buf_size >= needed:
             return output_buf.value
         else:
@@ -1247,22 +1558,21 @@ def getExternalUsePath(filename, only_dirname=False):
     Returns:
         Path that is a absolute and (on Windows) short filename pointing at the same file.
     Notes:
-        This is only "os.path.abspath" except on Windows, where is converts
+        This is only 'os.path.abspath' except on Windows, where is converts
         to a short path too.
     """
 
     filename = os.path.abspath(filename)
+    key = filename, only_dirname
 
-    if os.name == "nt":
-        key = filename, only_dirname
-
-        if key not in _external_use_path_cache:
+    if key not in _external_use_path_cache:
+        if os.name == "nt":
             filename = getFilenameRealPath(filename)
 
             if only_dirname:
                 dirname = getWindowsShortPathName(os.path.dirname(filename))
                 assert os.path.exists(dirname)
-                filename = os.path.join(dirname, os.path.basename(filename))
+                filename = getNormalizedPathJoin(dirname, os.path.basename(filename))
             else:
                 filename = getWindowsShortPathName(filename)
 
@@ -1272,17 +1582,24 @@ def getExternalUsePath(filename, only_dirname=False):
             # Looking the resolved path up again should give same result immediately.
             key = filename, only_dirname
             _external_use_path_cache[key] = filename
+        else:
+            _external_use_path_cache[key] = filename
 
-        return _external_use_path_cache[key]
-    else:
-        return filename
+    return _external_use_path_cache[key]
 
 
 _report_path_cache = {}
 
 
 def getReportSourceReference(source_ref):
-    """Convert a source ref into a path suitable for user output."""
+    """Convert a source ref into a path suitable for user output.
+
+    Args:
+        source_ref: Source reference object.
+
+    Returns:
+        str: String representation for reporting.
+    """
     return "%s:%s" % (
         getReportPath(source_ref.getFilename()),
         source_ref.getLineNumber(),
@@ -1290,7 +1607,15 @@ def getReportSourceReference(source_ref):
 
 
 def getReportPath(filename, prefixes=()):
-    """Convert filename into a path suitable for reporting, avoiding home directory paths."""
+    """Convert filename into a path suitable for reporting, avoiding home directory paths.
+
+    Args:
+        filename: Path to convert.
+        prefixes: Optional prefixes to relativize against.
+
+    Returns:
+        str: Path suitable for reporting.
+    """
     key = filename, tuple(prefixes)
 
     if key not in _report_path_cache:
@@ -1304,6 +1629,12 @@ def isNonLocalPath(path):
 
     This is not reliable and mainly for reporting purposes to identify paths
     work looking to abbreviate.
+
+    Args:
+        path: Path to check.
+
+    Returns:
+        bool: True if path is likely non-local.
     """
     return path.startswith("..") or os.path.isabs(path)
 
@@ -1321,22 +1652,16 @@ def _getReportPath(filename, prefixes):
             if isFilenameBelowPath(
                 path=prefix_path, filename=abs_filename, consider_short=False
             ):
-                return os.path.normpath(
-                    os.path.join(
-                        prefix_name, relpath(path=abs_filename, start=prefix_path)
-                    )
+                return getNormalizedPathJoin(
+                    prefix_name, relpath(path=abs_filename, start=prefix_path)
                 )
 
             if isFilenameBelowPath(
                 path=prefix_path, filename=abs_filename, consider_short=True
             ):
-                return os.path.normpath(
-                    os.path.join(
-                        prefix_name,
-                        relpath(
-                            path=abs_filename, start=getExternalUsePath(prefix_path)
-                        ),
-                    )
+                return getNormalizedPathJoin(
+                    prefix_name,
+                    relpath(path=abs_filename, start=getExternalUsePath(prefix_path)),
                 )
 
     if isWin32Windows():
@@ -1353,7 +1678,9 @@ def _getReportPath(filename, prefixes):
                 except FileNotFoundError:
                     pass
                 else:
-                    filename = os.path.join(dirname, os.path.basename(filename))
+                    filename = getNormalizedPathJoin(
+                        dirname, os.path.basename(filename)
+                    )
         else:
             if old_filename != filename:
                 return _getReportPath(filename, prefixes)
@@ -1377,7 +1704,7 @@ def getLinkTarget(filename):
     while os.path.exists(filename) and os.path.islink(filename):
         link_target = os.readlink(filename)
 
-        filename = os.path.join(os.path.dirname(filename), link_target)
+        filename = getNormalizedPathJoin(os.path.dirname(filename), link_target)
         is_link = True
 
     return is_link, filename
@@ -1441,7 +1768,12 @@ def resolveShellPatternToFilenames(pattern):
 
 @contextmanager
 def withDirectoryChange(path, allow_none=False):
-    """Change current directory temporarily in a context."""
+    """Change current directory temporarily in a context.
+
+    Args:
+        path: Director to be currently in.
+        allow_none: If None is passed, do nothing associated with this.
+    """
 
     # spell-checker: ignore chdir
 
@@ -1456,7 +1788,15 @@ def withDirectoryChange(path, allow_none=False):
 
 
 def containsPathElements(path, elements):
-    """Test if a path contains any unwanted elements."""
+    """Test if a path contains any unwanted elements.
+
+    Args:
+        path: directory path to check
+        elements: iterable of elements that must not be in the path
+
+    Returns:
+        bool - True if any of elements is part of the path name
+    """
 
     elements = tuple(os.path.normcase(element) for element in elements)
     path = os.path.normpath(path)
@@ -1508,6 +1848,14 @@ def isFilesystemEncodable(filename):
 
 
 def makeFilesystemEncodable(filename):
+    """Make a filename safe for filesystem usage.
+
+    Args:
+        filename: Path to make safe.
+
+    Returns:
+        str: Safe filename (short path on Windows if necessary).
+    """
     if not os.path.isabs(filename):
         filename = os.path.abspath(filename)
 
@@ -1518,6 +1866,16 @@ def makeFilesystemEncodable(filename):
 
 
 def openPickleFile(filename, mode, protocol=-1):
+    """Open a pickle file with proper handling.
+
+    Args:
+        filename: Path to pickle file.
+        mode: Open mode.
+        protocol: Pickle protocol version.
+
+    Returns:
+        tuple: (file_handle, pickler)
+    """
     file_handle = openTextFile(filename, mode)
 
     if python_version < 0x300:
@@ -1529,6 +1887,14 @@ def openPickleFile(filename, mode, protocol=-1):
 
 
 def isLegalPath(path):
+    """Check if a path is legal on the current platform.
+
+    Args:
+        path: Path to check.
+
+    Returns:
+        tuple: (bool, str or None) - (is_legal, illegal_reason)
+    """
     illegal_suffixes = "/\\"
     illegal_chars = "\0"
 
@@ -1576,7 +1942,14 @@ def isLegalPath(path):
 
 
 def getParentDirectories(path):
-    """Get all parent directories of a path in descending order."""
+    """Get all parent directories of a path in descending order.
+
+    Args:
+        path: Path to start from.
+
+    Yields:
+        str: Parent directory paths.
+    """
 
     while 1:
         old_path = path
@@ -1597,13 +1970,38 @@ def getNormalizedPath(path):
     if isWin32Windows():
         path = path.replace("/", "\\")
 
-    if isMacOS():
-        path = os.path.expanduser(path)
-
     return path
 
 
+def getUserInputNormalizedPath(path):
+    result = getNormalizedPath(path)
+
+    if isMacOS():
+        result = os.path.expanduser(result)
+
+    return result
+
+
+def getNormalizedPathJoin(*paths):
+    """Return join of path elements as a normalized path that is also a native path,
+       i.e. only legal characters.
+
+    Needed, because MSYS2 likes to keep "/" in normalized paths.
+    """
+    return getNormalizedPath(os.path.join(*paths))
+
+
 def getFileContentsHash(filename, as_string=True, line_filter=None):
+    """Get the hash of a file's content.
+
+    Args:
+        filename: Path to file.
+        as_string: Return as string (hex digest) or bytes (digest).
+        line_filter: Optional filter function for lines.
+
+    Returns:
+        str or bytes: Hash result.
+    """
     result = Hash()
     result.updateFromFile(filename=filename, line_filter=line_filter)
 
@@ -1613,14 +2011,33 @@ def getFileContentsHash(filename, as_string=True, line_filter=None):
         return result.asDigest()
 
 
+def encodeToFilesystemEncoding(path):
+    """Encode a path to filesystem encoding.
+
+    Args:
+        path: Path to encode.
+
+    Returns:
+        Encoded path (bytes on Python 2, str on Python 3).
+
+    Raises:
+        UnicodeEncodeError: If the path cannot be encoded in the filesystem encoding.
+    """
+    if str is bytes and type(path) is unicode:
+        # spell-checker: ignore getfilesystemencoding
+        return path.encode(sys.getfilesystemencoding() or "utf8")
+
+    return path
+
+
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     Licensed under the Apache License, Version 2.0 (the "License");
+#     Licensed under the GNU Affero General Public License, Version 3 (the "License");
 #     you may not use this file except in compliance with the License.
 #     You may obtain a copy of the License at
 #
-#        http://www.apache.org/licenses/LICENSE-2.0
+#        http://www.gnu.org/licenses/agpl.txt
 #
 #     Unless required by applicable law or agreed to in writing, software
 #     distributed under the License is distributed on an "AS IS" BASIS,
