@@ -1,9 +1,7 @@
 #     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
-""" Make a star import, and demonstrate that it has impact on the modules dict.
-
-"""
+"""Make a star import, and demonstrate that it has impact on the modules dict."""
 
 from __future__ import print_function
 
